@@ -78,7 +78,6 @@ void nsMsgHdr::Init()
   m_date = LL_ZERO;
   m_flags = 0;
   m_mdbRow = NULL;
-  m_numReferences = 0;
   m_threadId = nsMsgKey_None;
   m_threadParent = nsMsgKey_None;
 }
@@ -335,13 +334,11 @@ NS_IMETHODIMP nsMsgHdr::GetNumReferences(PRUint16 *result)
     if (NS_SUCCEEDED(m_mdb->RowCellColumnToConstCharPtr(GetMDBRow(),
                        m_mdb->m_referencesColumnToken, &references)))
       ParseReferences(references);
-    else
-      m_numReferences = 0;
     m_initedValues |= REFERENCES_INITED;
   }
 
   if (result)
-    *result = m_numReferences;
+    *result = m_references.Length();
   // there is no real failure here; if there are no references, there are no
   //  references.
   return NS_OK;
@@ -360,9 +357,8 @@ nsresult nsMsgHdr::ParseReferences(const char *references)
                                     startNextRef == references);
     // Don't add self-references.
     if (!resultReference.IsEmpty() && !resultReference.Equals(messageId))
-      m_references.AppendCString(resultReference);
+      m_references.AppendElement(resultReference);
   }
-  m_numReferences = m_references.Count();
   return NS_OK;
 }
 
@@ -373,8 +369,8 @@ NS_IMETHODIMP nsMsgHdr::GetStringReference(PRInt32 refNum, nsACString& resultRef
   if(!(m_initedValues & REFERENCES_INITED))
     GetNumReferences(nsnull); // it can handle the null
 
-  if (refNum < m_numReferences)
-    m_references.CStringAt(refNum, resultReference);
+  if ((PRUint32)refNum < m_references.Length())
+    resultReference = m_references.ElementAt(refNum);
   else
     err = NS_ERROR_ILLEGAL_VALUE;
   return err;
@@ -419,13 +415,8 @@ NS_IMETHODIMP nsMsgHdr::SetAuthor(const char *author)
 NS_IMETHODIMP nsMsgHdr::SetReferences(const char *references)
 {
   NS_ENSURE_ARG_POINTER(references);
-  if (*references == '\0') {
-    m_numReferences = 0;
-  }
-  else {
-    m_references.Clear();
-    ParseReferences(references);
-  }
+  m_references.Clear();
+  ParseReferences(references);
 
   m_initedValues |= REFERENCES_INITED;
 
@@ -633,7 +624,7 @@ NS_IMETHODIMP nsMsgHdr::GetAccountKey(char **aResult)
 }
 
 
-NS_IMETHODIMP nsMsgHdr::GetMessageOffset(PRUint32 *result)
+NS_IMETHODIMP nsMsgHdr::GetMessageOffset(PRUint64 *result)
 {
   NS_ENSURE_ARG(result);
 
@@ -643,7 +634,7 @@ NS_IMETHODIMP nsMsgHdr::GetMessageOffset(PRUint32 *result)
   GetRawFlags(&rawFlags);
   if (rawFlags & nsMsgMessageFlags::Offline)
   {
-    return GetUInt32Column(m_mdb->m_offlineMsgOffsetColumnToken, result);
+    return GetUInt64Column(m_mdb->m_offlineMsgOffsetColumnToken, result);
   }
   else
   {
@@ -652,9 +643,9 @@ NS_IMETHODIMP nsMsgHdr::GetMessageOffset(PRUint32 *result)
   }
 }
 
-NS_IMETHODIMP nsMsgHdr::SetMessageOffset(PRUint32 offset)
+NS_IMETHODIMP nsMsgHdr::SetMessageOffset(PRUint64 offset)
 {
-  SetUInt32Column(offset, m_mdb->m_offlineMsgOffsetColumnToken);
+  SetUInt64Column(offset, m_mdb->m_offlineMsgOffsetColumnToken);
   return NS_OK;
 }
 
@@ -733,17 +724,17 @@ NS_IMETHODIMP nsMsgHdr::GetMime2DecodedRecipients(nsAString &resultRecipients)
 }
 
 
-NS_IMETHODIMP nsMsgHdr::GetAuthorCollationKey(PRUint8 **resultAuthor, PRUint32 *len)
+NS_IMETHODIMP nsMsgHdr::GetAuthorCollationKey(PRUint32 *len, PRUint8 **resultAuthor)
 {
   return m_mdb->RowCellColumnToAddressCollationKey(GetMDBRow(), m_mdb->m_senderColumnToken, resultAuthor, len);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetSubjectCollationKey(PRUint8 **resultSubject, PRUint32 *len)
+NS_IMETHODIMP nsMsgHdr::GetSubjectCollationKey(PRUint32 *len, PRUint8 **resultSubject)
 {
   return m_mdb->RowCellColumnToCollationKey(GetMDBRow(), m_mdb->m_subjectColumnToken, resultSubject, len);
 }
 
-NS_IMETHODIMP nsMsgHdr::GetRecipientsCollationKey(PRUint8 **resultRecipients, PRUint32 *len)
+NS_IMETHODIMP nsMsgHdr::GetRecipientsCollationKey(PRUint32 *len, PRUint8 **resultRecipients)
 {
   return m_mdb->RowCellColumnToCollationKey(GetMDBRow(), m_mdb->m_recipientsColumnToken, resultRecipients, len);
 }
@@ -811,6 +802,16 @@ nsresult nsMsgHdr::GetUInt32Column(mdb_token token, PRUint32 *pvalue, PRUint32 d
   return m_mdb->RowCellColumnToUInt32(GetMDBRow(), token, pvalue, defaultValue);
 }
 
+nsresult nsMsgHdr::SetUInt64Column(PRUint64 value, mdb_token token)
+{
+  return m_mdb->UInt64ToRowCellColumn(m_mdbRow, token, value);
+}
+
+nsresult nsMsgHdr::GetUInt64Column(mdb_token token, PRUint64 *pvalue, PRUint64 defaultValue)
+{
+  return m_mdb->RowCellColumnToUInt64(GetMDBRow(), token, pvalue, defaultValue);
+}
+
 /**
  * Roughly speaking, get the next message-id (starts with a '<' ends with a
  *  '>').  Except, we also try to handle the case where your reference is of
@@ -958,7 +959,7 @@ NS_IMETHODIMP nsMsgHdr::GetIsRead(PRBool *isRead)
   NS_ENSURE_ARG_POINTER(isRead);
   if (!(m_initedValues & FLAGS_INITED))
     InitFlags();
-  *isRead = m_flags & nsMsgMessageFlags::Read;
+  *isRead = !!(m_flags & nsMsgMessageFlags::Read);
   return NS_OK;
 }
 
@@ -967,7 +968,7 @@ NS_IMETHODIMP nsMsgHdr::GetIsFlagged(PRBool *isFlagged)
   NS_ENSURE_ARG_POINTER(isFlagged);
   if (!(m_initedValues & FLAGS_INITED))
     InitFlags();
-  *isFlagged = m_flags & nsMsgMessageFlags::Marked;
+  *isFlagged = !!(m_flags & nsMsgMessageFlags::Marked);
   return NS_OK;
 }
 
@@ -1032,6 +1033,8 @@ PRBool nsMsgHdr::IsAncestorKilled(PRUint32 ancestorsToCheck)
       NS_ERROR("Thread is parent of itself, please fix!");
       nsCOMPtr<nsIMsgThread> thread;
       (void) m_mdb->GetThreadContainingMsgHdr(this, getter_AddRefs(thread));
+      if (!thread)
+        return PR_FALSE;
       ReparentInThread(thread);
       // Something's wrong, but the problem happened some time ago, so erroring
       // out now is probably not a good idea. Ergo, we'll pretend to be OK, show
